import numpy as np
import RobotDART as rd
import dartpy

# Load a robot
robot = rd.Franka()
robot.set_actuator_types("servo")

# set initial joint positions
positions = robot.positions()
positions[5] = np.pi / 2.0
positions[7] = 0.04
robot.set_positions(positions)

# Create simulator object
simu = rd.RobotDARTSimu(0.01)
# we need the FCL collision detector for mesh collisions
simu.set_collision_detector("fcl")

# Create Graphics
gconfig = rd.gui.GraphicsConfiguration(1024, 768) # Create a window of 1024x768 resolution/size
graphics = rd.gui.Graphics(gconfig) # create graphics object with configuration
simu.set_graphics(graphics)
graphics.look_at([3., 1., 2.], [0., 0., 0.])

# Add robot and nice floor
simu.add_robot(robot)
simu.add_checkerboard_floor()

# Run simulation
while True:
    if simu.step_world():
        break

    print('Mass matrix:\n', robot.mass_matrix())
    print('Inverse mass matrix:\n', robot.inv_mass_matrix())
    print('Coriolis:', robot.coriolis_forces())
    print('Gravity:', robot.gravity_forces())
    print('Coriolis/gravity:', robot.coriolis_gravity_forces())
