import numpy as np
import RobotDART as rd
import dartpy
import copy
from utils import angle_wrap_multi

# Class for PI controller in joint-space (works for 1-D or N-D vectors)
class PIJoint:
    def __init__(self, target, dt, Kp = 10., Ki = 0.1):
        self._target = target
        self._dt = dt
        self._Kp = Kp
        self._Ki = Ki
        self._sum_error = 0.

    def set_target(self, target):
        self._target = target

    def update(self, current):
        error = angle_wrap_multi(self._target - current) # since we have angles, it's better to wrap into [-pi,pi)
        self._sum_error = self._sum_error + error * self._dt

        return self._Kp * error + self._Ki * self._sum_error

# Load simple arm
robot = rd.Arm()
robot.set_position_enforced(True)
# select servo actuators --> velocity commands
robot.set_actuator_types("servo")

# Create simulator object
dt = 0.001
simu = rd.RobotDARTSimu(dt)

# Create Graphics
gconfig = rd.gui.GraphicsConfiguration(1024, 768) # Create a window of 1024x768 resolution/size
graphics = rd.gui.Graphics(gconfig) # create graphics object with configuration
simu.set_graphics(graphics)
graphics.look_at([3., 1., 2.], [0., 0., 0.])

# Add robot and nice floor
simu.add_robot(robot)
simu.add_checkerboard_floor()

Kp = 1.
Ki = 0.01
# 1-joint controller
# controller1 = PIJoint(1.2, dt, Kp, Ki)
# dof_control = 1
# dof_name = robot.dof_names()[dof_control]

# Multi-joint controller
target_positions = copy.copy(robot.positions())
target_positions[0] = 1.2
target_positions[2] = -1.
target_positions[3] = 0.5
controller = PIJoint(target_positions, dt, Kp, Ki) # Kp,Ki could be a numpy array (one for each dim)

# Run simulation
while True:
    if simu.step_world():
        break
    # v = controller1.update(robot.positions()[1])
    # robot.set_commands([v], [dof_name])
    cmd = controller.update(robot.positions())
    robot.set_commands(cmd)

